"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserHelperService = void 0;
const tslib_1 = require("tslib");
// Copyright (c) 2023 Sourcefuse Technologies
//
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT
const context_1 = require("@loopback/context");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const core_2 = require("@sourceloop/core");
const bcrypt = tslib_1.__importStar(require("bcrypt"));
const models_1 = require("../models");
const keys_1 = require("../keys");
const repositories_1 = require("../repositories");
const saltRounds = 10;
let UserHelperService = class UserHelperService {
    constructor(userRepository, userCredentialsRepository, getOtpRepository, userTenantRepositoryGetter, logger, passwordDecryptionFn, passwordHashingProvider) {
        this.userRepository = userRepository;
        this.userCredentialsRepository = userCredentialsRepository;
        this.getOtpRepository = getOtpRepository;
        this.userTenantRepositoryGetter = userTenantRepositoryGetter;
        this.logger = logger;
        this.passwordDecryptionFn = passwordDecryptionFn;
        this.passwordHashingProvider = passwordHashingProvider;
    }
    async create(entity, options) {
        const user = await this.userRepository.create(entity, options);
        try {
            // Add temporary password for first time
            const password = await this.passwordHashingProvider(process.env.USER_TEMP_PASSWORD);
            const creds = new models_1.UserCredentials({
                userId: user.id,
                authProvider: 'internal',
                password,
            });
            await this.userCredentialsRepository.create(creds);
        }
        catch (err) {
            throw new rest_1.HttpErrors.UnprocessableEntity('Error while hashing password');
        }
        return user;
    }
    async createWithoutPassword(entity, options) {
        return this.userRepository.create(entity, options);
    }
    async verifyPassword(username, password) {
        let newPassword = password;
        if (process.env.PRIVATE_DECRYPTION_KEY) {
            const decryptedPassword = await this.passwordDecryptionFn(password);
            newPassword = decryptedPassword;
        }
        const user = await this.userRepository.findOne({
            where: { username: username.toLowerCase() },
        });
        const creds = user &&
            (await this.userCredentialsRepository.findOne({
                where: { userId: user.id },
            }));
        if (!user || user.deleted) {
            throw new rest_1.HttpErrors.Unauthorized("UserDoesNotExist" /* AuthenticateErrorKeys.UserDoesNotExist */);
        }
        else if (!(creds === null || creds === void 0 ? void 0 : creds.password) ||
            creds.authProvider !== core_2.AuthProvider.INTERNAL ||
            !(await bcrypt.compare(newPassword, creds.password))) {
            this.logger.error('User creds not found in DB or is invalid');
            throw new rest_1.HttpErrors.Unauthorized("Invalid Credentials" /* AuthErrorKeys.InvalidCredentials */);
        }
        else {
            return user;
        }
    }
    async decryptPassword(password) {
        const decryptedPassword = await this.passwordDecryptionFn(password);
        return decryptedPassword;
    }
    async updatePassword(username, password, newPassword) {
        var _a;
        const user = await this.userRepository.findOne({ where: { username } });
        const creds = user &&
            (await this.userCredentialsRepository.findOne({
                where: { userId: user.id },
            }));
        if ((_a = (!user || user.deleted)) !== null && _a !== void 0 ? _a : !(creds === null || creds === void 0 ? void 0 : creds.password)) {
            throw new rest_1.HttpErrors.Unauthorized("UserDoesNotExist" /* AuthenticateErrorKeys.UserDoesNotExist */);
        }
        else if (creds.authProvider !== core_2.AuthProvider.INTERNAL) {
            throw new rest_1.HttpErrors.BadRequest("PasswordCannotBeChangedForExternalUser" /* AuthenticateErrorKeys.PasswordCannotBeChanged */);
        }
        else if (!(await bcrypt.compare(password, creds.password))) {
            throw new rest_1.HttpErrors.Unauthorized("Incorrect Password" /* AuthErrorKeys.WrongPassword */);
        }
        else if (await bcrypt.compare(newPassword, creds.password)) {
            throw new rest_1.HttpErrors.Unauthorized('Password cannot be same as previous password!');
        }
        else {
            // Do nothing
        }
        creds.password = await bcrypt.hash(newPassword, saltRounds);
        await this.userCredentialsRepository.update(creds);
        return user;
    }
    async changePassword(username, newPassword, oldPassword) {
        var _a;
        const user = await this.userRepository.findOne({ where: { username } });
        const creds = user &&
            (await this.userCredentialsRepository.findOne({
                where: { userId: user.id },
            }));
        if (oldPassword) {
            // This method considers old password as OTP
            const otp = await (await this.getOtpRepository()).get(username);
            if (!otp || otp.otp !== oldPassword) {
                throw new rest_1.HttpErrors.Unauthorized("Incorrect Password" /* AuthErrorKeys.WrongPassword */);
            }
        }
        if ((creds === null || creds === void 0 ? void 0 : creds.authProvider) !== core_2.AuthProvider.INTERNAL) {
            throw new rest_1.HttpErrors.Unauthorized("PasswordCannotBeChangedForExternalUser" /* AuthenticateErrorKeys.PasswordCannotBeChanged */);
        }
        if ((_a = (!user || user.deleted)) !== null && _a !== void 0 ? _a : !(creds === null || creds === void 0 ? void 0 : creds.password)) {
            throw new rest_1.HttpErrors.Unauthorized("UserDoesNotExist" /* AuthenticateErrorKeys.UserDoesNotExist */);
        }
        else if (await bcrypt.compare(newPassword, creds.password)) {
            throw new rest_1.HttpErrors.Unauthorized('Password cannot be same as previous password!');
        }
        else {
            // DO nothing
        }
        creds.password = await bcrypt.hash(newPassword, saltRounds);
        await this.userCredentialsRepository.update(creds);
        return user;
    }
    async updateLastLogin(userId) {
        await this.userRepository.updateById(userId, {
            lastLogin: Date.now(),
        }, {
            currentUser: { id: userId },
        });
    }
    async firstTimeUser(userId) {
        const user = await this.userRepository.findOne({
            where: {
                id: userId,
            },
        });
        if (!user) {
            throw new rest_1.HttpErrors.NotFound("UserDoesNotExist" /* AuthenticateErrorKeys.UserDoesNotExist */);
        }
        const userTenant = await (await this.userTenantRepositoryGetter()).findOne({
            where: {
                userId,
                tenantId: user.defaultTenantId,
                status: {
                    inq: [0 /* UserStatus.REGISTERED */, 3 /* UserStatus.PASSWORD_CHANGE_NEEDED */],
                },
            },
        });
        return !!userTenant;
    }
};
exports.UserHelperService = UserHelperService;
exports.UserHelperService = UserHelperService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.TRANSIENT }),
    tslib_1.__param(0, (0, repository_1.repository)(repositories_1.UserRepository)),
    tslib_1.__param(1, (0, repository_1.repository)(repositories_1.UserCredentialsRepository)),
    tslib_1.__param(2, repository_1.repository.getter(repositories_1.OtpRepository)),
    tslib_1.__param(3, repository_1.repository.getter('UserTenantRepository')),
    tslib_1.__param(4, (0, context_1.inject)(core_2.LOGGER.LOGGER_INJECT)),
    tslib_1.__param(5, (0, context_1.inject)(keys_1.AuthServiceBindings.PASSWORD_DECRYPTION_PROVIDER)),
    tslib_1.__param(6, (0, context_1.inject)(keys_1.AuthServiceBindings.PASSWORD_HASHING_PROVIDER)),
    tslib_1.__metadata("design:paramtypes", [repositories_1.UserRepository,
        repositories_1.UserCredentialsRepository, Function, Function, Object, Function, Function])
], UserHelperService);
//# sourceMappingURL=user-helper.service.js.map